/**
 *  InternalCertificate.java
 *  Description: Internal Certificate Manipulation
 *
 *  @author Dohan Kim <kimw@uwindsor.ca>
 *  @version 1.0
 *  @date Feb. 2003
 */

package dhkim.ca;

import java.security.*;
import java.security.cert.*;
import java.io.FileInputStream;
import java.io.Serializable;
import java.io.FileNotFoundException;
import javax.swing.JTextArea;
import java.util.*;

public class InternalCertificate
{

 /**
  * Method importCertificate
  * Description: Import the certificate.
  *
  * @return X509 Certificate.
  *
  */

  public static X509Certificate importCertificate(
    char[] keystore, char[] keystore_password, char[] key_alias )
  {
    java.security.cert.Certificate cert = null;
    try
    {
      FileInputStream inputStream = new FileInputStream( String.valueOf( keystore ) );
      KeyStore keyStore = KeyStore.getInstance("JKS");
      keyStore.load( inputStream , keystore_password );
      inputStream.close();
      cert = keyStore.getCertificate( String.valueOf( key_alias ) );
    }
    catch( Exception e )
    {
      e.printStackTrace();
    }
    return ( X509Certificate ) cert;
  }

  public static X509Certificate importCertificate( String filename )
  {
    X509Certificate cert = null;
    try
    {
      CertificateFactory cf =
        CertificateFactory.getInstance( "X509" );
      /**
       * Get the File I/O of the Certificate
       */
      FileInputStream fr = new FileInputStream( filename );
      /**
       *  Construct the certificate based on the import
       */
      cert = (X509Certificate)cf.generateCertificate( fr );
    }
    catch( CertificateException e )
    {
      e.printStackTrace();
    }
    catch( FileNotFoundException e )
    {
      e.printStackTrace();
    }
    return cert;
  }

  /**
   * Method printCertificate
   * Description: Print information of the Certificate.
   *
   * @param cert is the certification to read from.
   *
   */

  public static void printCertificate( X509Certificate cert, JTextArea outputArea )
  {
    try
    {
      outputArea.append("***************************************************");
      outputArea.append( "\n" );
      outputArea.append( cert.toString());
      outputArea.append( "**************************************************" );
      outputArea.append( "\n\n" );
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
  }

  public static PrivateKey getPrivateKey(
    char[] keystore, char[] keystore_password, char[] alias, char[] alias_password )
  {
    FileInputStream inputStream = null;
    KeyStore keyStore = null;
    PrivateKey private_key = null;
    try
    {
      inputStream = new FileInputStream( String.valueOf( keystore ) );
      keyStore = KeyStore.getInstance("JKS");
      keyStore.load( inputStream , keystore_password );
      inputStream.close();
      private_key =
        (PrivateKey)keyStore.getKey( String.valueOf( alias ), alias_password );
    }
    catch( Exception e )
    {
      e.printStackTrace();
    }
    return private_key;
  }

  public static void viewAllCertificates
    ( JTextArea textarea, char[] keystore, char[] keystore_password )
  {
    try
    {
      FileInputStream inStream = new FileInputStream( String.valueOf( keystore ) );
      KeyStore keyStore = KeyStore.getInstance( "JKS" );
      keyStore.load( inStream , keystore_password );
      inStream.close();
      for (Enumeration enum = keyStore.aliases() ; enum.hasMoreElements() ;)
      {
        X509Certificate cert =
          ( X509Certificate ) keyStore.getCertificate( (String)enum.nextElement() );
        printCertificate( cert, textarea );
      }
    }
    catch ( Exception ex )
    {
      ex.printStackTrace();
    }
  }

  public static X509Certificate getSelectedCertificate(
    String alias, char[] keystore, char[] keystore_password )
  {
    X509Certificate cert = null;
    try
    {
      FileInputStream in = new FileInputStream( String.valueOf( keystore ) );
      KeyStore keyStore = KeyStore.getInstance( "JKS" );
      keyStore.load( in , keystore_password );
      in.close();
      cert = ( X509Certificate ) keyStore.getCertificate( alias );
    }
    catch( Exception e )
    {
      e.printStackTrace();
    }
    return cert;
  }
}
