/**
 *  InternalCRL.java
 *
 *  Description: Internal CRL Manipulation
 *
 *  @author Dohan Kim <kimw@uwindsor.ca>
 *  @version 1.0
 *  @date Feb. 2003
 */

package dhkim.ca;

import java.io.*;
import java.security.*;
import java.security.cert.*;
import java.util.*;
import javax.swing.JTextArea;
import sun.security.x509.*;

public class InternalCRL
{
  private static JTextArea outputArea = null;
  private static X500Name issuer = null;
  private static PrivateKey caPrivateKey = null;
  /**
   *  Validity is set to one month
   */
  private static final int VALIDITY = 30;
  private static String CRL_FILE_NAME = "InternalCA.crl";

  public InternalCRL ( JTextArea outputArea, X500Name issuer , PrivateKey caPrivateKey )
  {
    this.issuer = issuer;
    this.outputArea = outputArea;
    this.caPrivateKey = caPrivateKey;
  }

  public static void updateCRL( X509Certificate revoke_certificate )
  {
    try
    {
      int i = 0;
      X509CRL crl = importCRL( CRL_FILE_NAME );
      Date update = new Date();
      Date next_update = new Date(update.getTime() + VALIDITY*24*60*60*1000L);
      X509CRLEntryImpl entry = new X509CRLEntryImpl(
        revoke_certificate.getSerialNumber(), new Date(), null );
      Set setCRLEntries = crl.getRevokedCertificates();
      X509CRLEntry[] crl_entries = null ;
      if( setCRLEntries == null )
      {
        crl_entries = new X509CRLEntry[ 1 ];
        crl_entries[ 0 ] = ( X509CRLEntry )entry;
      }

      else
      {
        crl_entries = new X509CRLEntry[ setCRLEntries.size() + 1 ];
        for( Iterator iterator = setCRLEntries.iterator(); iterator.hasNext(); )
        {
          crl_entries[ i++ ] = ( X509CRLEntry )iterator.next();
        }
        crl_entries[ i ] = ( X509CRLEntry )entry;
      }
      X509CRLImpl updateCRL = new X509CRLImpl( issuer, update, next_update,
        crl_entries, null );
      updateCRL.sign( caPrivateKey, "MD5WithRSA" );
      FileOutputStream output = new FileOutputStream( "InternalCA.crl" );
      output.write( updateCRL.getEncoded() );
      output.close();
    }
    catch( Exception e )
    {
      e.printStackTrace();
    }
  }

  public static void generateCRL()
  {
    try
    {
      Date update_date = new Date();
      Date next_update_date = new Date(update_date.getTime() + VALIDITY*24*60*60*1000L);
      X509CRLImpl internalCRL = new X509CRLImpl( issuer, update_date, next_update_date, null, null );
      internalCRL.sign( caPrivateKey, "MD5WithRSA" );
      FileOutputStream outputStream = new FileOutputStream( "InternalCA.crl" );
      outputStream.write( internalCRL.getEncoded() );
      outputStream.close();
    }
    catch ( Exception ce )
    {
      ce.printStackTrace();
    }
  }

  public static void viewCRL( JTextArea outputArea )
  {
    try
    {
      X509CRL tmp_crl = importCRL( CRL_FILE_NAME );
      outputArea.append("***************************************************");
      outputArea.append( "\n" );
      outputArea.append( tmp_crl.toString() );
      outputArea.append( "\n" );
      outputArea.append("***************************************************");
    }
    catch( Exception e )
    {
      e.printStackTrace();
    }
  }

  public static X509CRL importCRL( String filename )
  {
    X509CRL crl_cert = null;
    try
    {
      CertificateFactory cf =
        CertificateFactory.getInstance( "X509" );
      FileInputStream fr = new FileInputStream( filename );
      crl_cert = ( X509CRL )cf.generateCRL( fr );
      fr.close();
    }
    catch( Exception e )
    {
      e.printStackTrace();
    }
    return crl_cert;
  }
}
