/**
 *  InternalCAImpl.java
 *
 *  Description: Internal CA Implementation
 *
 *  @author Dohan Kim <kimw@uwindsor.ca>
 *  @version 1.0
 *  @date Feb. 2003
 */

package dhkim.caserver;

import java.rmi.server.*;
import java.rmi.*;
import java.util.*;
import java.io.*;
import java.security.*;
import java.security.cert.*;
import java.util.*;
import java.math.BigInteger;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.io.InputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import sun.security.x509.*;
import dhkim.ca.*;

public class InternalCAImpl extends UnicastRemoteObject implements InternalCA
{
  private static Vector unsigned_server_list = new Vector();
  private InternalServerData internalserverdata = null;
  private static int num = 0;
  private JTextArea unsigned_textarea = new JTextArea();
  private JTextArea signed_textarea = new JTextArea();
  private JTextArea crl_textarea = new JTextArea();
  private JTabbedPane tabbedpane = null;
  public JFrame frame = null;
  public JComboBox view_combobox = null;
  public JComboBox sign_combobox = null;
  public JComboBox signed_view_combobox = null;
  private JTextField revoke_textfield = new JTextField( 12 );
  private char[] CA_KEYSTORE_NAME = "castore".toCharArray();
  private char[] CA_KEYSTORE_PASSWORD = "caserver".toCharArray();
  private char[] CA_ALIAS_NAME = "ca".toCharArray();
  private char[] CA_ALIAS_PASSWORD = "caserver".toCharArray();
  private char[] SERVER_ALIAS_NAME = null;
  private String CRL_FILE_NAME = "InternalCA.crl";
  private JPasswordField passwordfield = new JPasswordField();
  private InternalCAAuth ca_auth = null;
  private InternalSignCertificate internal_sign_certificate = null;
  private InternalCRL internal_crl = null;
  static int revoke_index = 0;

  public InternalCAImpl() throws RemoteException
  {
    drawUI();
    ca_auth = new InternalCAAuth();
    internal_sign_certificate = new InternalSignCertificate
      ( unsigned_textarea, CA_KEYSTORE_NAME, CA_KEYSTORE_PASSWORD,
      CA_ALIAS_NAME, CA_ALIAS_PASSWORD );
    internal_crl = new InternalCRL( crl_textarea ,
      internal_sign_certificate.getIssuer(),
      InternalCertificate.getPrivateKey( CA_KEYSTORE_NAME, CA_KEYSTORE_PASSWORD,
      CA_ALIAS_NAME, CA_ALIAS_PASSWORD) );
    internal_crl.generateCRL();
  }

  public void drawUI()
  {
    String id[] = { "SELECT", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10" };
    JMenuBar menubar = new JMenuBar();
    JMenu filemenu = new JMenu( "File" );
    JMenu toolmenu = new JMenu( "Tools" );
    JMenu helpmenu = new JMenu( "Help" );
    menubar.add( filemenu );
    menubar.add( toolmenu );
    menubar.add( helpmenu );
    tabbedpane = new JTabbedPane();
    JPanel unsigned_cert_list_panel = new JPanel();
    unsigned_cert_list_panel.setLayout( new BorderLayout() );
    unsigned_textarea.append(" Internal CA Started" + "\n\n" );
    unsigned_textarea.setEditable( false );
    JPanel button_panel = new JPanel();
    JLabel view_label = new JLabel( "VIEW :" );
    view_combobox = new JComboBox( id );
    view_combobox.setMaximumRowCount( 11 );
    view_combobox.addItemListener(
      new ItemListener()
      {
        public void itemStateChanged( ItemEvent e )
        {
          int num = view_combobox.getSelectedIndex();
          internalserverdata =
            ( InternalServerData )unsigned_server_list.elementAt( num - 1 );
          InternalCertificate.printCertificate
            ( internalserverdata.getcert(), unsigned_textarea );
        }
      }
    );
    JLabel sign_label = new JLabel( "SIGN :" );
    sign_combobox = new JComboBox( id );
    sign_combobox.setMaximumRowCount( 10 );
    sign_combobox.addItemListener(
      new ItemListener()
      {
        public void itemStateChanged( ItemEvent e )
        {
          int will_be_signed = sign_combobox.getSelectedIndex();
          internalserverdata =
            ( InternalServerData )unsigned_server_list.elementAt( num - 1 );
          X509Certificate will_be_signed_cert
            = internalserverdata.getcert();
          try
          {
            internal_sign_certificate.SignCertifcate
              ( will_be_signed_cert.getEncoded(), SERVER_ALIAS_NAME );
          }
          catch( CertificateException ce)
          {
            ce.printStackTrace();
          }
        }
      }
    );
    button_panel.setLayout( new FlowLayout() );
    button_panel.add( view_label );
    button_panel.add( view_combobox );
    button_panel.add( sign_label );
    button_panel.add( sign_combobox );
    JScrollPane scrollpane = new JScrollPane( unsigned_textarea );
    unsigned_cert_list_panel.add( scrollpane, BorderLayout.CENTER );
    unsigned_cert_list_panel.add( button_panel, BorderLayout.SOUTH );

    JPanel signed_cert_list_panel = new JPanel();
    signed_cert_list_panel.setLayout( new BorderLayout() );
    signed_textarea.setEditable( false );
    JPanel signed_button_panel = new JPanel();
    JLabel view_all_label = new JLabel(" View All Signed Certificate : ");
    JButton view_all_button = new JButton( "VIEW ALL" );
    view_all_button.addActionListener(
      new ActionListener(){
        public void actionPerformed(ActionEvent e){
          InternalCertificate.viewAllCertificates(
            signed_textarea, CA_KEYSTORE_NAME, CA_KEYSTORE_PASSWORD );
      }
    } );
    signed_button_panel.setLayout( new FlowLayout() );
    signed_button_panel.add( view_all_label );
    signed_button_panel.add( view_all_button );
    JScrollPane signed_scrollpane = new JScrollPane( signed_textarea );
    signed_cert_list_panel.add( signed_scrollpane, BorderLayout.CENTER );
    signed_cert_list_panel.add( signed_button_panel, BorderLayout.SOUTH );

    JPanel crl_list_panel = new JPanel();
    crl_list_panel.setLayout( new BorderLayout() );
    crl_textarea.setEditable( false );
    JPanel crl_button_panel = new JPanel();
    JLabel revoke_cert_label = new JLabel( "Choose Certificate(alias_name) :" );
    JButton revoke_button = new JButton( "Revoke" );
    revoke_button.addActionListener(
      new ActionListener(){
        public void actionPerformed(ActionEvent e){
        try
        {
          X509Certificate cert =
            InternalCertificate.getSelectedCertificate(
              revoke_textfield.getText(), CA_KEYSTORE_NAME, CA_KEYSTORE_PASSWORD );
          System.out.println( revoke_textfield.getText() );
          System.out.println( cert.toString() );
          internal_crl.updateCRL( cert  );
        }
        catch ( Exception e2 ){
          e2.printStackTrace();
        }
      }
    } );

    JButton cer_view_button = new JButton( "VIEW CERTIFICATES" );
    cer_view_button.addActionListener(
      new ActionListener(){
        public void actionPerformed(ActionEvent e){
        try {
         InternalCertificate.viewAllCertificates( crl_textarea, CA_KEYSTORE_NAME, CA_KEYSTORE_PASSWORD );
        }
        catch ( Exception e3 ){
          e3.printStackTrace();
        }
      }
    } );

    JLabel crl_view_label = new JLabel(" View CRL : ");
    JButton crl_view_button = new JButton( "VIEW CRL" );
    crl_view_button.addActionListener(
      new ActionListener(){
        public void actionPerformed(ActionEvent e){
        try {
          internal_crl.viewCRL( crl_textarea );
        }
        catch ( Exception e4 ){
          e4.printStackTrace();
        }
      }
    } );

    crl_button_panel.setLayout( new FlowLayout() );
    crl_button_panel.add( cer_view_button );
    crl_button_panel.add( revoke_cert_label );
    crl_button_panel.add( revoke_textfield );
    crl_button_panel.add( revoke_button );
    crl_button_panel.add( crl_view_label );
    crl_button_panel.add( crl_view_button );
    JScrollPane crl_scrollpane = new JScrollPane( crl_textarea );
    crl_list_panel.add( crl_scrollpane, BorderLayout.CENTER );
    crl_list_panel.add( crl_button_panel, BorderLayout.SOUTH );

    tabbedpane.addTab( "Unsigned Certificate", unsigned_cert_list_panel );
    tabbedpane.addTab( "Signed Certificate", signed_cert_list_panel );
    tabbedpane.addTab( "CRL", crl_list_panel );

    frame = new JFrame( "592 Project : Internal CA" );
    frame.setJMenuBar( menubar );
    frame.getContentPane().add( tabbedpane, BorderLayout.CENTER );
    frame.setSize( 800, 600 );
    frame.setVisible( true );
    frame.addWindowListener( new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        System.exit(1);
      }
    });
  }

  public void registerServer( String server_alias, byte[] certbuff )
  {
    try
    {
      SERVER_ALIAS_NAME = server_alias.toCharArray();
      X509CertImpl serverCert = new X509CertImpl( certbuff );
      num++;
      internalserverdata = new InternalServerData( num, server_alias, ( X509Certificate )serverCert );
      unsigned_server_list.addElement( internalserverdata );
      show_Unsigned_Certificate_List();
    }
    catch( Exception e )
    {
      e.printStackTrace();
    }
  }

  public void show_Unsigned_Certificate_List ()
  {
    for( Enumeration e = unsigned_server_list.elements(); e.hasMoreElements(); )
    {
      try
      {
        unsigned_textarea.append( "**********************************************" + "\n" );
        InternalServerData data = ( InternalServerData )e.nextElement();
        unsigned_textarea.append( "NO . " + String.valueOf( data.getnum() ) + "\n" );
        unsigned_textarea.append( "Server Alias of CA KeyStore :  " + String.valueOf( data.getAlias() ) + "\n" );
        unsigned_textarea.append( "**********************************************" + "\n" );
      }
      catch( NoSuchElementException ne )
      {
        ne.printStackTrace();
      }
    }
  }

  public byte[] getCACertificate( String ca_alias )
  {
    X509Certificate cacert = null;
    byte[] ca_encoded = null;
    try
    {
      cacert = InternalCertificate.getSelectedCertificate(
        ca_alias, CA_KEYSTORE_NAME, CA_KEYSTORE_PASSWORD );
      ca_encoded = cacert.getEncoded();
    }
    catch( Exception e )
    {
      e.printStackTrace();
    }
    return ca_encoded;
  }

  public byte[] getServerCertificate( String server_alias )
  {
    X509Certificate server_cert = null;
    byte[] server_encoded = null;
    try
    {
      server_cert = InternalCertificate.getSelectedCertificate(
        server_alias, CA_KEYSTORE_NAME, CA_KEYSTORE_PASSWORD );
      server_encoded = server_cert.getEncoded();
    }
    catch( Exception e )
    {
      e.printStackTrace();
    }
    return server_encoded;
  }

  public static void main( String args[] )
  {
    try
    {
      InternalCAImpl internalcaimpl = new InternalCAImpl();
      String registry = "localhost";
      if( args.length >= 1 )
      {
        registry = args[0];
      }
      String registration = "rmi://" + registry + "/InternalCA";
      Naming.rebind( registration, internalcaimpl);
    }
    catch( RemoteException re )
    {
      re.printStackTrace();
    }
    catch( Exception e )
    {
      e.printStackTrace();
    }
  }
}