/**
 *  InternalSignCertificate.java
 *  Description : Changing the field of Certificate and Signing the Certificate.
 */

package dhkim.caserver;

import java.io.*;
import java.security.*;
import java.security.cert.*;
import java.util.*;
import javax.swing.JTextArea;
import sun.security.x509.*;
import dhkim.ca.*;

public class InternalSignCertificate
{
  /**
   *  Algorithm to use for signing the certificate
   */
  private static final String SIG_ALGORITHM = "MD5WithRSA";
  /**
   *  Validity for the certificate, in days
   */
  private static final int VALIDITY = 365;
  private Date firstDate = new Date();
  public char[] CA_KEYSTORE_NAME = null;
  public char[] CA_KEYSTORE_PASSWORD = null;
  public char[] CA_ALIAS_NAME = null;
  public char[] CA_ALIAS_PASSWORD = null;
  public char[] SERVER_ALIAS_NAME = null;
  public JTextArea outputArea = null;
  private X500Name issuer = null;
  private PrivateKey caPrivateKey = null;
  private X509CertImpl caCertImpl = null;
  private X509CertInfo caCertInfo = null;

  public InternalSignCertificate( JTextArea outputArea, char[] CA_KEYSTORE_NAME,
    char[] CA_KEYSTORE_PASSWORD, char[] CA_ALIAS_NAME, char[] CA_ALIAS_PASSWORD )
  {
    this.outputArea = outputArea;
    this.CA_KEYSTORE_NAME = CA_KEYSTORE_NAME;
    this.CA_KEYSTORE_PASSWORD = CA_KEYSTORE_PASSWORD;
    this.CA_ALIAS_NAME = CA_ALIAS_NAME;
    this.CA_ALIAS_PASSWORD = CA_ALIAS_PASSWORD;
    initialize();
  }

  public void initialize()
  {
    try
    {
      /**
       *  Getting the CA's private key for signing
       */
      caPrivateKey = InternalCertificate.getPrivateKey(
        CA_KEYSTORE_NAME, CA_KEYSTORE_PASSWORD, CA_ALIAS_NAME, CA_ALIAS_PASSWORD );
      /**
       *  Getting the CA's certificate
       */
      X509Certificate caCert = InternalCertificate.importCertificate(
        CA_KEYSTORE_NAME, CA_KEYSTORE_PASSWORD, CA_ALIAS_NAME );
      byte[] encoded = caCert.getEncoded();
      caCertImpl = new X509CertImpl( encoded );
      caCertInfo = ( X509CertInfo )caCertImpl.get
        (X509CertImpl.NAME + "." + X509CertImpl.INFO);
      issuer = ( X500Name )caCertInfo.get
        (X509CertInfo.SUBJECT + "." + CertificateIssuerName.DN_NAME);
    }
    catch( Exception e )
    {
      e.printStackTrace();
    }
  }

  public void SignCertifcate( byte[] will_be_signed_cert, char[] server_alias_name )
  {
    try
    {
      this.SERVER_ALIAS_NAME = server_alias_name;
      /**
       *  Getting the server which will be signed certificate by byte[] array of parameters
       */
      X509CertImpl certImpl = new X509CertImpl( will_be_signed_cert );
      X509CertInfo certInfo = (X509CertInfo)certImpl.get
        (X509CertImpl.NAME + "." + X509CertImpl.INFO);
      /**
       *  Setting the validity
       */
      Date lastDate = new Date(firstDate.getTime() + VALIDITY*24*60*60*1000L);
      CertificateValidity interval = new CertificateValidity(firstDate, lastDate);
      certInfo.set( X509CertInfo.VALIDITY, interval );
      /**
       *  Making a new serial number
       */
      certInfo.set(X509CertInfo.SERIAL_NUMBER,
        new CertificateSerialNumber( ( int )( firstDate.getTime()/1000 ) ) );
      /**
       *  Setting the issuer
       */
      certInfo.set(X509CertInfo.ISSUER + "." +
        CertificateSubjectName.DN_NAME, issuer);
      AlgorithmId algorithm = new AlgorithmId
        ( AlgorithmId.md5WithRSAEncryption_oid );
      certInfo.set( CertificateAlgorithmId.NAME + "." +
        CertificateAlgorithmId.ALGORITHM, algorithm );
      X509CertImpl cert_to_be_stored = new X509CertImpl( certInfo );
      /**
       *  Signing the certificate
       */
      System.out.println( " Key Format: " + caPrivateKey.getFormat() );
      cert_to_be_stored.sign( caPrivateKey, SIG_ALGORITHM );
      X509Certificate certificate = ( X509CertImpl )cert_to_be_stored;
      /**
       *  Storing signed certificate into the CA Keystore
       */
      FileInputStream inputStream =
        new FileInputStream( String.valueOf( CA_KEYSTORE_NAME ) );
      KeyStore keyStore = KeyStore.getInstance("JKS");
      keyStore.load( inputStream , CA_KEYSTORE_PASSWORD );
      inputStream.close();
      keyStore.setCertificateEntry( String.valueOf( SERVER_ALIAS_NAME ), certificate );
      FileOutputStream output =
        new FileOutputStream( String.valueOf( CA_KEYSTORE_NAME ) );
      keyStore.store( output, CA_KEYSTORE_PASSWORD );
      output.close();
      InternalCertificate.printCertificate( certificate, outputArea );
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  public X500Name getIssuer()
  {
    return issuer;
  }
}

