/**
 *  InternalCAClient.java
 *  Description: Internal Demo CA Client
 *
 *  @author Dohan Kim <kimw@uwindsor.ca>
 *  @version 1.0
 *  @date Feb. 2003
 */

package dhkim.client;

import java.rmi.*;
import java.util.*;
import java.security.*;
import java.security.cert.*;
import java.math.BigInteger;
import sun.security.x509.*;
import dhkim.ca.*;

public class InternalCAClient
{
  private static String CRL_FILE_NAME = "InternalCA.crl";
  private static String SIG_ALG_NAME = "MD5WithRSA";
  private PublicKey ca_pubkey = null;
  private PublicKey server_pubkey = null;

  public boolean verifyCRL( byte[] server_cert )
  {
    X509CertImpl serverCertificate = null;
    try
    {
      serverCertificate = new X509CertImpl( server_cert );
      BigInteger serial = serverCertificate.getSerialNumber();
      X509CRL crl = InternalCRL.importCRL( CRL_FILE_NAME );
      Set setCRLEntries = crl.getRevokedCertificates();
      /**
       * Checking if the certificate serial number is in the crl list
       */
      if( setCRLEntries != null && setCRLEntries.isEmpty() == false )
      {
        for(Iterator iterator = setCRLEntries.iterator(); iterator.hasNext(); )
        {
          X509CRLEntry entry = ( X509CRLEntry )iterator.next();
          if( serial.compareTo( entry.getSerialNumber() ) == 0 )
          {
            return false;
          }
        }
      }
    }
    catch( Exception e )
    {
      e.printStackTrace();
    }
    return true;
  }

  public PublicKey getCAPubKey()
  {
    return ca_pubkey;
  }

  public PublicKey getServerPubKey()
  {
    return server_pubkey;
  }

  public boolean verifyServer( byte[] caCert, byte[] serverCert )
  {
    X509CertImpl cacert = null;
    X509CertImpl server_cert = null;
    try
    {
      cacert = new X509CertImpl( caCert );
      server_cert = new X509CertImpl( serverCert );
      ca_pubkey = cacert.getPublicKey();
      server_pubkey = server_cert.getPublicKey();
      System.out.println(
         "Verifying the CA Certificate Using Public Key Of CA" );
      cacert.verify( ca_pubkey );
      System.out.println( "Verifying of CA Certificate Succeeded" );
      System.out.println( "Verifying of CA CRL List..." + "\n" );
      boolean bool = verifyCRL( serverCert );
      if( bool == true )
        System.out.println( "Succeeded : Verifying of CA CRL List." );
      else
      {
        System.out.println( "Failure : Verifying of CA CRL List." );
        System.exit( -1 );
      }
      System.out.println( "Verifying of CA CRL List..." );
      System.out.println(
        "Verifying the Server Certificate Using Public Key Of CA" );
      server_cert.verify( ca_pubkey );
    }
    catch( Exception e )
    {
      e.printStackTrace();
    }
    return true;
  }

  public static void main( String args[] )
  {
    InternalCA internalca = null;
    InternalServer internalserver = null;
    InternalCAClient internal_client = null;
    InternalCAInput input = new InternalCAInput();
    try
    {
      internal_client = new InternalCAClient();
      String registry = "localhost";
      if( args.length >= 1 )
      {
        registry = args[0];
      }
      String ca_registration = "rmi://" + registry + "/InternalCA";
      String server_registration = "rmi://" + registry + ":1234" + "/InternalServer";
      Remote remoteCA = Naming.lookup( ca_registration );
      internalca = (InternalCA )remoteCA;
      Remote remoteServer = Naming.lookup( server_registration );
      internalserver = (InternalServer )remoteServer;

      System.out.print( "Enter the CA Name for getting the CA Certificate : " );
      String caname = input.readLine();
      System.out.println( "\n" );
      System.out.println( "Receiving the CA Certificate from CA........ " );
      byte[] caCert = internalca.getCACertificate( caname );
      System.out.println( new X509CertImpl( caCert ).toString() );
      System.out.println("Done.... " + "\n" );

      System.out.print( "Enter the Server Name for getting the Server Certificate: " );
      String servername = input.readLine();
      System.out.println( "Receiving the Server Certificate from .. " + servername );
      byte[] serverCert = internalca.getServerCertificate( servername );
      System.out.println( new X509CertImpl( serverCert ).toString() );
      System.out.println( "Done " + "\n" );

      System.out.println( "Verifying the Server Certificate:" );
      boolean b = internal_client.verifyServer( caCert, serverCert );
      if( b == true )
        System.out.println( "Succeeded : Verified the Server Certificate" );
      else
      {
        System.out.print( "Failure of  Verifying the Server Certificate:" );
        System.exit( -1 );
      }

      System.out.print( "Enter the message for Server verifying : ");
      String challenge = input.readLine();
      System.out.println( "Client Sending message ..: " + challenge );
      byte[] response = internalserver.verifyRequest( challenge.trim() , SIG_ALG_NAME );
      System.out.println( "Client received signature : " + response.toString() );
      byte[] querry = challenge.trim().getBytes();
      Signature sig = Signature.getInstance( SIG_ALG_NAME );
      sig.initVerify( internal_client.getServerPubKey() );
      sig.update( querry, 0, querry.length );
      if( sig.verify( response ) == true )
      {
        System.out.println( "Retrieved Server Public Key From CA " +
          new X509CertImpl( serverCert ).getPublicKey().toString() );
        System.out.println( "Verified Message From Server Public Key From CA ." );
      }
      else
      {
        System.out.println( "Failure of Verifying." );
       }
    }
    catch( Exception e )
    {
      e.printStackTrace();
    }
  }
}





