/**
 *  InternalServerImpl.java
 *
 *  Description: Internal Server Implementation
 *
 *  @author Dohan Kim <kimw@uwindsor.ca>
 *  @version 1.0
 *  @date Feb. 2003
 */

package dhkim.server;

import java.rmi.server.*;
import java.rmi.*;
import java.util.*;
import java.security.cert.*;
import java.security.*;
import dhkim.ca.*;

public class InternalServerImpl extends UnicastRemoteObject implements InternalServer
{
  private char[] SERVER_KEYSTORE_NAME = "fileserver".toCharArray();
  private char[] SERVER_KEYSTORE_PASSWORD = "fileserver".toCharArray();
  private char[] SERVER_ALIAS_NAME = "fileserver".toCharArray();
  private char[] SERVER_ALIAS_PASSWORD = "fileserver".toCharArray();

  public InternalServerImpl() throws RemoteException
  {
  }

  public byte[] verifyRequest( String message, String sig_alg_name )
  {
    System.out.println( "Received message from client: " + message );
    byte[] response = null ;
    try
    {
      PrivateKey server_priv = InternalCertificate.getPrivateKey(
        SERVER_KEYSTORE_NAME, SERVER_KEYSTORE_PASSWORD,
        SERVER_ALIAS_NAME, SERVER_ALIAS_PASSWORD );
      Signature rsa_sig = Signature.getInstance( sig_alg_name );
      rsa_sig.initSign( server_priv );
      byte[] buff = message.getBytes();
      rsa_sig.update( buff, 0, buff.length );
      response = rsa_sig.sign();
      System.out.println( " Done. " );
    }
    catch( Exception e )
    {
      e.printStackTrace();
    }
    return response ;
  }

  public static void main( String args[] )
  {
    InternalCAInput input = new InternalCAInput();
    InternalCA internalca = null ;
    try
    {
      InternalServerImpl internalserverimpl = new InternalServerImpl();
      String registry = "localhost";
      if( args.length >= 1 )
      {
        registry = args[0];
      }
      String registration = "rmi://" + registry + ":1234" + "/InternalServer";
      Naming.rebind( registration, internalserverimpl);

      String ca_registration = "rmi://" + registry + "/InternalCA";
      Remote remoteCA = Naming.lookup( ca_registration );
      internalca = (InternalCA )remoteCA;
      /**
       * Server Name will be the alias of Certificate name of CA Key Store
       */
      System.out.print( "Enter the Server Name : ");
      String servername = input.readLine();
      System.out.println("");
      System.out.print( "Enter the Certificate File Name : ");
      String filename = input.readLine();
      System.out.println("");
      System.out.println( filename );
      X509Certificate cert = InternalCertificate.importCertificate( filename );
      /**
       * Changes the encoded form of this certificate.
       * It is assumed that each certificate type would have only a single form of encoding;
       * for example, X.509 certificates would be encoded as ASN.1 DER.
       */
      System.out.println( "Server Certificate" + "\n" + cert.toString() + "\n\n" );
      System.out.println( "Server Public Key : " + cert.getPublicKey().toString() );
      internalca.registerServer( servername, cert.getEncoded() );
    }
    catch( NotBoundException nbe )
    {
      nbe.printStackTrace();
    }
    catch( RemoteException re )
    {
      re.printStackTrace();
    }
    catch( Exception e )
    {
      e.printStackTrace();
    }
  }
}